{
Knihovna urcena k mereni prubehu casu vykonavani funkci

Namety k rozsireni
- dle vrchni pozice zasobniku by sla zachytavat trasa, kterou program pri prubehu prochazel
- mohli bychom pocitat, kolikrat byla ktera funkce spustena
- mohli bychom pocitat, kolikrat byla ktera funkce inkrementovana (coz je rozdil oproti spusteni. k inkrementaci dochazi vzhledem k zanorovani profilovanych funkci mnohem casteji)
- do vizualizeru udelat poradnou metodu pro vypis peknych vysledku mereni


Autori:
  Medwyn_cz (napad, specifikace, implementace)
  Gord (specifikace)
  
1.0.0
1.0.1 Gord (22.12.2014)
      - doplnena kontrola prekroceni zasobniku a hloubky zanoreni
      - doplnen vypis vrchu zasobniku pri prekroceni maximalniho zanoreni

}

//Na konci knihovny je includovan visualizer

const
  MAX_PROFILED_ON_TIME = 100; //Maximalni pocet do sebe zanorenych profilovanych funkci
  STORAGE_SIZE = 1000; //Maximalni pocet ruznych profilovanych funkci. Velikost skladiste
  PROFILE_ISINIT = 'Y'; //Pomocna konstanta pro kontrolu stavu inicializace

type
  TStack=array[0..MAX_PROFILED_ON_TIME] of Integer; //Typ zasobniku indexu profilovanych funkci
  TTimeStorage=array[0..STORAGE_SIZE] of LongWord; //Typ skladiste namerenych hodnot 

var
  stack : TStack; //Zasobnik indexu profilovanych funkci 
  storage : TTimeStorage; //Skladiste namerenych hodnot
  lexicon : TStrings; //Skladiste jmen profilovanych funkci
  profilerInit : String; //Byla knihovna inicializovana? pokud ano, obsahuje PROFILE_ISINIT
  lastTick : LongWord; //Posledni namereny cas
  stackPosition : Integer; //Pozice prave zpracovavaneho prvku v zasobniku

{Byl beh knihovny inicializovan?}
function Initiated : boolean;
begin
   Result := (profilerInit = PROFILE_ISINIT);
end;

{Nastavit informaci o inicializaci behu knihovny}
procedure SetInitiated(bool : boolean);
begin
   if bool 
    then profilerInit := PROFILE_ISINIT
    else profilerInit := 'Neinicializovano';
end;

{Probihaji prave nejaka mereni?}
function MeasuringStarted: boolean;
begin
   Result := (stackPosition > -1);
end;

{Inicializacni procedura profileru. Neni nutno ji volat rucne, ale je to mozne}
procedure ProfileInit;
var n : Integer;
begin
  if not Initiated then
  begin
    lexicon := TStringList.Create;
    stackPosition := -1;
    for n := 0 to STORAGE_SIZE do storage[n] := 0;
    SetInitiated(true);
  end
end;

{Uklizeci procedura profileru. Neni nutne ji na konci volat}
procedure ProfileFree;
begin
  if Initiated then
  begin
    lexicon.Free;
    SetInitiated(false);
  end; 
end;

{Vyvolat kritickou chybu a ukoncit prubeh}
procedure RaiseCriticalError(error : String);
begin
    ShowMessage(error);
    ProfileFree;
    GeoAbort;
end;

{Funkce vraci pozici daneho sampleName uvnitr skladiste namerenych hodnot. Pokud doposud neexistuje, zalozi mu pozici novou}
function GetStorageIndex(sampleName : String) : Integer;
begin
   if pos(CRLF,sampleName) > 0 then RaiseCriticalError(_('Do not try to add sample names with line breaks'));
   if pos('|',sampleName) > 0 then RaiseCriticalError(_('Do not try to add sample names with | in them'));
   Result := lexicon.IndexOf(sampleName);
   if (Result = -1) then Result := lexicon.Add(sampleName);
end;

{Procedura ukonci mereni, vypocte odpracovany cas a ulozi jej do skladiste}
procedure AddMeasuredTimeToStorage(storageIndex : Integer);
begin
   storage[storageIndex] := storage[storageIndex] + (GetTick - lastTick);
end;  

{Procedura zapocne mereni casu nove procedury. Pokud jiz nejaka predchozi procedura ma mereni casu spusteno, je toto nejprve ukonceno}
procedure ProfileStart(sampleName : String);
var s:string;
    i:integer;
begin
   if (profilerInit <> PROFILE_ISINIT) then ProfileInit;
   if(stackPosition>=MAX_PROFILED_ON_TIME) then begin
     s:='<star of stack>'+CRLF+'...'+CRLF;
     for i:=stackPosition-5 to stackPosition do s:=s+'    '+lexicon[stack[stackPosition]]+CRLF;
     s:=s+'<end of stack>'
     RaiseCriticalError(_('The number of nesting profiled function (MAX_PROFILED_ON_TIME) is exceeded.'+CRLF+s));
   end
   else begin
     if (stackPosition > -1) then
     begin
       AddMeasuredTimeToStorage(stack[stackPosition]);
     end;
     stackPosition := stackPosition + 1;
     stack[stackPosition] := GetStorageIndex(sampleName);
     lastTick := GetTick;
   end;
end;

{Procedura ukonci mereni casu predchozi procedury}
procedure ProfileStop;
begin
   if not MeasuringStarted then RaiseCriticalError(_('No profiling has been started. You must call ProfileStart first.'));
   AddMeasuredTimeToStorage(stack[stackPosition]);
   stackPosition := stackPosition - 1;
   lastTIck := GetTick;
end;

{Procedure vrati vysledky mereni ve tvaru samleName|namerenyCas}
function ProfileResults: TStrings;
var s : String;
    n : Integer;
begin
   if MeasuringStarted then 
     begin
        s := _('Running profilings still exist. Please, close all your ProfileStarts with ProfileStop! There are currently %RUNNING_PROFILINGS% nested profilings running, with "%TOP_PROFILING%" at the top.');
        s := ReplaceString(s,'%RUNNING_PROFILINGS%',IntToStr(stackPosition+1));
        s := ReplaceString(s,'%TOP_PROFILING%',lexicon[stack[stackPosition]]);
        RaiseCriticalError(s);
     end
   else
     begin
        Result := TStringList.Create;
        for n := 0 to lexicon.count-1 do
        begin
           Result.Add(lexicon[n] + '|' + IntToStr(storage[n]));        
        end;
        ProfileFree;     
     end;
end;

{$I Profile.visualizer.lib.pas}